# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

from requre.simple_object import Simple as _Simple
from requre.simple_object import Void as _Void
from requre.simple_object import Tuple as _Tuple

# BACKWARDS COMPATIBILITY
Simple = _Simple
Void = _Void
Tuple = _Tuple
